<?php

namespace Drupal\library\Helper;


use Drupal\library\Entity\LibraryAction;
use Drupal\library\Entity\LibraryItem;
use Drupal\node\Entity\Node;

class LibraryItemHelper {

  /**
   * @param $barcode
   * @return bool|LibraryItem
   */
  public static function findByBarcode($barcode) {
    $items = [];
    $results = \Drupal::entityQuery('library_item')
      ->condition('barcode', $barcode)
      ->execute();
    foreach ($results as $result) {
      $items[] = LibraryItem::load($result);
    }
    if (count($items) == 1 && $items[0] instanceof LibraryItem) {
      return $items[0];
    } else {
      return FALSE;
    }
  }

  /**
   * @param integer $item
   * @param string $action
   */
  public static function updateItemAvailability($item, $action) {
    $action = LibraryAction::load($action);
    $item = LibraryItem::load($item);
    if ($action->action() == LibraryAction::CHANGE_TO_UNAVAILABLE) {
      $item->set('library_status', LibraryItem::ITEM_UNAVAILABLE);
      $item->save();
    } else if ($action->action() == LibraryAction::CHANGE_TO_AVAILABLE) {
      $item->set('library_status', LibraryItem::ITEM_AVAILABLE);
      $item->save();
    }
  }

  /**
   * @param string $action
   * @param int $nid
   *
   * @return int
   *
   */
  public static function computeDueDate($action, $nid) {
    $action = LibraryAction::load($action);

    if ($action->action() != LibraryAction::CHANGE_TO_AVAILABLE) {
      $due = 30;
      $node = Node::load($nid);
      $bundle_fields = \Drupal::getContainer()->get('entity_field.manager')->getFieldDefinitions('node', $node->getType());
      foreach ($bundle_fields as $field) {
        if ($field->getType() == 'library_item_field_type') {
          $due = $field->getSetting('due_date');
        }
      }
      $due = strtotime('today') + (86400 * $due);
    } else {
      $due = 0;
    }
    return $due;
  }
}